(define-module controller
  (use srfi-13)
  (use srfi-27)
  (use rfc.uri) 
  (use www.cgi) 
  (use orm)
  (use user)
  (use template)
  (export dispatch))
(select-module controller)

(define *conts* (make-hash-table 'eqv?))
(define *max-cid* (expt 2 64))

(define (get-cont params)
  (hash-table-get *conts* 
		  (cgi-get-parameter "@cont@" params :convert string->number)
		  #f))

(define (push-cont! cont) 
  (let1 cid (random-integer *max-cid*)
    (cond ((hash-table-get *conts* cid #f) (push-cont! cont))
	  (else (hash-table-put! *conts* cid cont) cid))))

(define-macro (define-continuation args . body)
  `(push-cont! (lambda ,args ,@body)))

(define (dispatch request)
  (receive (authority path query fragment) (uri-decompose-hierarchical request) 
    (let1 params (cgi-parse-parameters :query-string (or query ""))
      (cond ((get-cont params) => (lambda(c) (c params)))
	    ((string= path "/edit") (edit params))
	    ((string= path "/update") (update params))
	    (else (index))))))

(define (index)
  (let1 players (all <player>)
    (render "index.tmpl"  players)))

(define (edit params)
  (let* ((player (find <player> 
		       (cgi-get-parameter "id" params :convert string->number)))
	 (cid (define-continuation (params)
		(set-params player params)
		(save player)
		(render "update.tmpl"))))
    (render "edit.tmpl"  player cid)))

(provide "controller")




